@@ -66,6 +66,7 @@ class TestCalendarSerializing(unittest.TestCase):
                               tzinfo=dateutil.tz.tzical(
                                   "test_files/timezones.ics").get('US/Pacific'))
         cal.vevent.add('uid').value = "Not very random UID"
+        cal.vevent.add('dtstamp').value = datetime.datetime(2017, 6, 26, 0, tzinfo=tzutc())
 
         # Note we're normalizing line endings, because no one got time for that.
         self.assertEqual(
@@ -94,7 +95,7 @@ class TestCalendarSerializing(unittest.TestCase):
             self.assertEqual(str(vevent), str(vevent2))
             self.assertEqual(
                 vevent.summary.value,
-                'The title こんにちはキティ'
+                u'The title こんにちはキティ'
             )
 
     def test_wrapping(self):
@@ -705,6 +706,7 @@ class TestIcalendar(unittest.TestCase):
 
         vfb = base.newFromBehavior('VFREEBUSY')
         vfb.add('uid').value = 'test'
+        vfb.add('dtstamp').value = datetime.datetime(2006, 2, 15, 0, tzinfo=utc)
         vfb.add('dtstart').value = datetime.datetime(2006, 2, 16, 1, tzinfo=utc)
         vfb.add('dtend').value   = vfb.dtstart.value + two_hours
         vfb.add('freebusy').value = [(vfb.dtstart.value, two_hours / 2)]
@@ -800,6 +802,29 @@ class TestIcalendar(unittest.TestCase):
             [datetime.datetime(2005, 3, 18, 0, 0), datetime.datetime(2005, 3, 29, 0, 0)]
         )
 
+    def test_recurrence_without_tz(self):
+        """
+        Test recurring vevent missing any time zone definitions.
+        """
+        test_file = get_test_file("recurrence-without-tz.ics")
+        cal = base.readOne(test_file)
+        dates = list(cal.vevent.getrruleset())
+        self.assertEqual(dates[0], datetime.datetime(2013, 1, 17, 0, 0))
+        self.assertEqual(dates[1], datetime.datetime(2013, 1, 24, 0, 0))
+        self.assertEqual(dates[-1], datetime.datetime(2013, 3, 28, 0, 0))
+
+    def test_recurrence_offset_naive(self):
+        """
+        Ensure recurring vevent missing some time zone definitions is
+        parsing. See isseu #75.
+        """
+        test_file = get_test_file("recurrence-offset-naive.ics")
+        cal = base.readOne(test_file)
+        dates = list(cal.vevent.getrruleset())
+        self.assertEqual(dates[0], datetime.datetime(2013, 1, 17, 0, 0))
+        self.assertEqual(dates[1], datetime.datetime(2013, 1, 24, 0, 0))
+        self.assertEqual(dates[-1], datetime.datetime(2013, 3, 28, 0, 0))
+
 
 class TestChangeTZ(unittest.TestCase):
     """
